function [] = pickPolygon(src, ~)
%PICKPOLYGON Lets the user pick a polygon in the plot.
%   During selection, a line is being plotted that shows the selected
%   points by the user.
%   The polygon can be stopped by using the right mouse button. If
%   finished, the selected points are being calculated and stored in the
%   artoaWorkspace variable.

global artoaGui artoaWorkspace;

modeName = 'pickPolygon';

%% Check for any active mode
if strcmp(artoaGui.editTimeOfArrival.currentInteractionMode, modeName)
    % pickPoint mode is active, so finish the mode
    closePickPolygonMode();
    return;
end

if ~islogical(artoaGui.editTimeOfArrival.currentInteractionMode)
    errordlg('Another interaction mode is currently active. Please finish it first!', 'Interaction mode error!');
    return;
end

%% Set mode to pickPoint
openPickPolygonMode();

%% Check if polygon is already selected
artoa.controller.edit.timeOfArrival.dropSelection();

%% Initialize variable
artoaGui.editTimeOfArrival.userSelection = {};
artoaWorkspace.editTimeOfArrival.userSelection = false(length(artoaWorkspace.toaData.toa), 1);

%% Initialize polygon
artoaGui.editTimeOfArrival.tmpPolygon = struct();
artoaGui.editTimeOfArrival.tmpPolygon.x = [];
artoaGui.editTimeOfArrival.tmpPolygon.y = [];

% draw the line that will be extended on every click
artoaGui.editTimeOfArrival.userSelection = line( ...
    NaN, NaN, 'Color', [0 0 0], 'LineStyle', '-' ...
);

button = src;

return

%% Helper functions

    function openPickPolygonMode()
        pan('off');
        zoom('off');
        brush('off');
        
        set(artoaGui.figures.editTimeOfArrival, 'WindowButtonMotionFcn', @onMouseMove);
        set(artoaGui.figures.editTimeOfArrival, 'WindowButtonDownFcn', @onMouseAxesDown);
        artoaGui.editTimeOfArrival.currentInteractionMode = modeName;
        artoaGui.figures.editTimeOfArrival.Color = [.6 .6 .6];
        src.String = 'Finish';
        drawnow;
    end

    function closePickPolygonMode()
        finishPolygon();
        set(artoaGui.figures.editTimeOfArrival, 'WindowButtonMotionFcn', '');
        set(artoaGui.figures.editTimeOfArrival, 'WindowButtonDownFcn', '');
        artoaGui.editTimeOfArrival.currentInteractionMode = false;
        artoaGui.figures.editTimeOfArrival.Color = [.94 .94 .94];
        src.String = 'Polygon';
        drawnow;
    end

    function onMouseAxesDown(src, ~)

        if strcmp(src.SelectionType, 'alt')
            closePickPolygonMode();
            return;
        end

        currentPoint = artoaGui.editTimeOfArrival.axesScatterTimeOfArrival.CurrentPoint;
        
        % get boundaries
        bound = artoaGui.editTimeOfArrival.axesScatterTimeOfArrival.XLim;
        dx = artoa.data.adjustToBoundaries(currentPoint(1, 1), bound(1), bound(2));
        bound = artoaGui.editTimeOfArrival.axesScatterTimeOfArrival.YLim;
        dy = artoa.data.adjustToBoundaries(currentPoint(1, 2), bound(1), bound(2));
        
        % add values to array
        artoaGui.editTimeOfArrival.tmpPolygon.x = [artoaGui.editTimeOfArrival.tmpPolygon.x; dx];
        artoaGui.editTimeOfArrival.tmpPolygon.y = [artoaGui.editTimeOfArrival.tmpPolygon.y; dy];
        
        button.String = ['Finish ' num2str(length(artoaGui.editTimeOfArrival.tmpPolygon.x)) ' selected'];
        
        if length(artoaGui.editTimeOfArrival.tmpPolygon.x) == 1
            return;
        end
        % update the line
        set( ...
            artoaGui.editTimeOfArrival.userSelection, ...
            'XData', artoaGui.editTimeOfArrival.tmpPolygon.x, ...
            'YData', artoaGui.editTimeOfArrival.tmpPolygon.y ...
            );
        
    end

    function onMouseMove(~, ~)
        if ~artoa.data.hasMember(artoaGui, 'editTimeOfArrival', 'tmpPolygon', 'x') ...
                || isempty(artoaGui.editTimeOfArrival.tmpPolygon.x)
            return;
        end
        
        mousepoint = get(artoaGui.editTimeOfArrival.axesScatterTimeOfArrival, 'CurrentPoint');
        set(artoaGui.editTimeOfArrival.userSelection, ...
            'XData', [artoaGui.editTimeOfArrival.tmpPolygon.x; mousepoint(1, 1) - .1], ...
            'YData', [artoaGui.editTimeOfArrival.tmpPolygon.y; mousepoint(1, 2) - .1] ...
            );
        drawnow;
    end

    function finishPolygon()
        % close the poligon by adding the first value as the last one
        if length(artoaGui.editTimeOfArrival.tmpPolygon.x) > 3 % a polygon can only be closed if there are more than three values
            x = [artoaGui.editTimeOfArrival.tmpPolygon.x; artoaGui.editTimeOfArrival.tmpPolygon.x(1)];
            y = [artoaGui.editTimeOfArrival.tmpPolygon.y; artoaGui.editTimeOfArrival.tmpPolygon.y(1)];
        else
            [artoaGui.editTimeOfArrival, artoaWorkspace.editTimeOfArrival] = artoa.controller.edit.clearSelection(artoaGui.editTimeOfArrival, artoaWorkspace.editTimeOfArrival);
            return;
        end
        % update the line
        set(artoaGui.editTimeOfArrival.userSelection, 'XData', x, 'YData', y);
        % store the selected points in the workspace for editing
        artoaWorkspace.editTimeOfArrival.userSelection = inpolygon( ...
            artoaWorkspace.toaData.toaDate, ...
            artoaWorkspace.toaData.toa, ...
            x, ...
            y ...
        );
    end

end

